/******************************************************************/
/*									                              */
/*      Description : Main process                                 */
/*      Making Date : 2010. 05. 06					              */
/*      Affiliate   : Machine Learning Lab in SoongSil Univ       */
/*      Developer   : Sungin Park(rfchief@gmail.com)              */
/******************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include "common.h"
#include "utilFunc.h"
#include "sampling.h"
#include "CalEvalues.h"
#include "FileWriter.h"

int run(struct paramsStr params);
void showHelp();
void printOut(struct paramsStr params);
void writeLog(struct paramsStr params);

int main(int argc, char** argv)
{
	char logMsg[1000];
	char fPath[1100];
	struct paramsStr params;

	if(argc > 1){
		if(parseParams(argc, argv, &params)){			
			writeLog(params);			
			run(params);	
	
			printf("\nprogram terminated\n");
		}else{			
			errorExit(-9);
		}
	}else{		
		showHelp();
	}

	sprintf(fPath, "%s_Log.txt", params.outName);
	sprintf(logMsg, "CFMDS program finishes");
	writeLogFile(fPath, logMsg, 0);	

	return 0;
}


int run(struct paramsStr params){
	int size     = params.size;
	char method  = params.method;
	char evChk   = params.svd;
	int pMax     = 0;
	int p        = decDivideCnt(size, &pMax, params);    		

	if(evChk == 'y' || evChk == 'Y'){
		printf("Calculate eigenvalues process starts\n\n");
		calEvaluesProcess(params, p, pMax);		
	}else{	
		if(p > 0){
			if(method == 'r' || method == 'R'){
				printf("The sampling strategy is Rnd\n");
				runRnd(params, p, pMax);						
			}else{
				printf("The sampling strategy is MaxMin\n");
				runMaxMin(params, p, pMax);				
			}
		}else{
			runMDS(params);					
		}//if(p>0)
	}	

	return 0;
}

void showHelp(){
	printf("CFMDS command line usage:\n");
	printf("==============================================================================\n\n");
	printf("-s <int>\n");
	printf("\tThe number of data points.\n");
	printf("\tEx : 3000, 4000, etc.\n\n");
	printf("-d <int>\n");
	printf("\tThe target dimension which is lower than the original data dimension.\n");
	printf("\tEx : 2, 4, etc.\n\n");
	printf("-h <char>\n");
	printf("\tInput data has title column and row (Yes or No).\n");
	printf("\tEx : Y, y, N, n (default : No)\n\n");
	printf("-e <char>\n");
	printf("\tCalculate eigenvalues only (Yes or No).\n");
	printf("\tEx : Y, y, N, n (default : No)\n\n");
	printf("-m <char>\n");
	printf("\tSampling strategy for divid-and-conquer mode (Mode 2).\n");
	printf("\tEx : R, r, M, m (default : Random)\n\n");
	printf("-o <string>\n");
	printf("\tPrefix of output files.\n");
	printf("\tEx : CFMDS_Result, IRIS_CFMDS, etc.\n\n");
	printf("-i <string>\n");
	printf("\tInput file name.\n");
	printf("\tEx : home/usr/3000.out, c:\\data\\iris.txt, test.txt, etc.\n");
	printf("\n==============================================================================\n");		
}

void printOut(struct paramsStr params){
	
	printf("Result of parsing of command parameters:\n");
	printf("=======================================\n\n");
	printf("Size : %d\n", params.size);
	printf("Dimension : %d\n", params.dimension);
	printf("Header : %c\n", params.header);
	printf("Method : %c\n", params.method);
	printf("SVD : %c\n", params.svd);
	printf("OutName : %s\n", params.outName);
	printf("InputName : %s\n", params.inputName);
	printf("\n=======================================\n");
	
}

void writeLog(struct paramsStr params){
	char logMsg[1100];
	char fPath[1100];

	sprintf(fPath, "%s_Log.txt", params.outName);
	sprintf(logMsg, "Result of command parameters parsing\n");
	if(writeLogFile(fPath, logMsg, 1)){
		sprintf(logMsg, "# of data points : %d", params.size);
		writeLogFile(fPath, logMsg, 0);
		sprintf(logMsg, "Output dimension : %d", params.dimension);
		writeLogFile(fPath, logMsg, 0);
		sprintf(logMsg, "Header : %c", params.header);
		writeLogFile(fPath, logMsg, 0);
		sprintf(logMsg, "Sampling method : %c", params.method);
		writeLogFile(fPath, logMsg, 0);
		sprintf(logMsg, "Eigenvalue Calculation : %c", params.svd);
		writeLogFile(fPath, logMsg, 0);
		sprintf(logMsg, "Output prefix : %s", params.outName);
		writeLogFile(fPath, logMsg, 0);
		sprintf(logMsg, "Input file : %s", params.inputName);
		writeLogFile(fPath, logMsg, 0);				
		writeLogFile(fPath, logMsg, 2);				
	}else{
		printf("Log file write error\n");
		
		sprintf(fPath, "CFMDS_Log.txt");
		sprintf(logMsg, "Log file write error.");
		writeLogFile(fPath, logMsg, 1);
		sprintf(logMsg, "Program terminated.");
		writeLogFile(fPath, logMsg, 0);

		errorExit(4);
	}	
}
