/******************************************************************/
/*									                              */
/*      Description : CUDA Memory testing process                 */
/*      Making Date : 2010. 05. 06					              */
/*      Affiliate   : Machine Learning Lab in SoongSil Univ       */
/*      Developer   : Sungin Park(rfchief@gmail.com)              */
/******************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <cuda.h>
#include "memTest.h"
#include "FileWriter.h"
#include "common.h"

unsigned long inKB(unsigned long bytes)
{ return bytes/1024; }

unsigned long inMB(unsigned long bytes)
{ return bytes/(1024*1024); }

int chkMem(struct paramsStr params)
{
  unsigned int free, total;
  int freeMemMB, totalMemMB;
  int gpuCount, i;
  CUresult res;
  CUdevice dev;
  CUcontext ctx;
  char logMsg[1000];
  char fPath[1100];

  cuInit(0);

  cuDeviceGetCount(&gpuCount);
  printf("%d GPU(s) detected\n",gpuCount);
  
  sprintf(fPath, "%s_Log.txt", params.outName);
  sprintf(logMsg, "GPU memory test process\n",gpuCount);
  writeLogFile(fPath, logMsg, 0);
  sprintf(logMsg, "%d GPU(s) detected",gpuCount);
  writeLogFile(fPath, logMsg, 0);

  for (i=0; i<gpuCount; i++)
  {
	  cuDeviceGet(&dev,i);
	  cuCtxCreate(&ctx, 0, dev);
	  res = cuMemGetInfo(&free, &total);
	  if(res != CUDA_SUCCESS){
		  printf("!!!! cuMemGetInfo failed! (status = %x)", res);
		  sprintf(logMsg, "!!!! cuMemGetInfo failed! (status = %x)", res);
		  writeLogFile(fPath, logMsg, 0);
	  }
	  printf("Device: %d\n",i);
	  
	  freeMemMB  = (int)inMB(free);
	  totalMemMB = (int)inMB(total);

	  printf("Free : %d MB\n", freeMemMB);
      printf("Total: %d MB\n", totalMemMB);
	  printf("%3.2f%% free, %3.2f%% used\n\n", 100.0*free/(double)total, 100.0*(total - free)/(double)total);
	  
	  sprintf(logMsg, "Free : %d MB", freeMemMB);
	  writeLogFile(fPath, logMsg, 0);
      sprintf(logMsg, "Total: %d MB", totalMemMB);
	  writeLogFile(fPath, logMsg, 0);
	  sprintf(logMsg, "%3.2f%% free, %3.2f%% used\n", 100.0*free/(double)total, 100.0*(total - free)/(double)total);
	  writeLogFile(fPath, logMsg, 0);	  

	  cuCtxDetach(ctx);
  }

  return freeMemMB;
}
